/*
 * IRC - Internet Relay Chat, ircd/m_addoper.c
 * Copyright (C) 2003 Inter System
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_addoper.c,v 1.1 2003/02/20 16:51:16 progs Exp $
 */

/*
 * m_functions execute protocol messages on this server:
 *
 *    cptr    is always NON-NULL, pointing to a *LOCAL* client
 *            structure (with an open socket connected!). This
 *            identifies the physical socket where the message
 *            originated (or which caused the m_function to be
 *            executed--some m_functions may call others...).
 *
 *    sptr    is the source of the message, defined by the
 *            prefix part of the message if present. If not
 *            or prefix not found, then sptr==cptr.
 *
 *            (!IsServer(cptr)) => (cptr == sptr), because
 *            prefixes are taken *only* from servers...
 *
 *            (IsServer(cptr))
 *                    (sptr == cptr) => the message didn't
 *                    have the prefix.
 *
 *                    (sptr != cptr && IsServer(sptr) means
 *                    the prefix specified servername. (?)
 *
 *                    (sptr != cptr && !IsServer(sptr) means
 *                    that message originated from a remote
 *                    user (not local).
 *
 *            combining
 *
 *            (!IsServer(sptr)) means that, sptr can safely
 *            taken as defining the target structure of the
 *            message in this server.
 *
 *    *Always* true (if 'parse' and others are working correct):
 *
 *    1)      sptr->from == cptr  (note: cptr->from == cptr)
 *
 *    2)      MyConnect(sptr) <=> sptr == cptr (e.g. sptr
 *            *cannot* be a local connection, unless it's
 *            actually cptr!). [MyConnect(x) should probably
 *            be defined as (x == x->from) --msa ]
 *
 *    parc    number of variable parameter strings (if zero,
 *            parv is allowed to be NULL)
 *
 *    parv    a NULL terminated list of parameter pointers,
 *
 *                    parv[0], sender (prefix string), if not present
 *                            this points to an empty string.
 *                    parv[1]...parv[parc-1]
 *                            pointers to additional parameters
 *                    parv[parc] == NULL, *always*
 *
 *            note:   it is guaranteed that parv[0]..parv[parc-1] are all
 *                    non-NULL pointers.
 */
#include "config.h"

#include "client.h"
#include "msg.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_features.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "numeric.h"
#include "numnicks.h"
#include "s_conf.h"
#include "send.h"

#include <assert.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

/*
 * mo_addoper - oper message handler
 *
 * parv[0] = sender
 * parv[1] = login
 * parv[2] = host
 * parv[3] = oflags
 * parv[4] = pass
 */
int mo_addoper(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
	char oline[512];
	int cfd, arf;

	if(!IsAnAdmin(sptr))
		return send_reply(sptr, ERR_NOPRIVILEGES);

	if(parc < 5)
	{
		need_more_params(sptr, "ADDOPER");
		sendcmdto_one(&me, CMD_NOTICE, sptr,
		  "%C :Syntax : ADDOPER <login> <host> <flags> <pass>", sptr);
		return 0;
	}

	strcpy(oline,"O:");
	strncat(oline,parv[2],256);
	strcat(oline,":");
	strncat(oline,parv[4],32);
	strcat(oline,":");
	strncat(oline,parv[1],32);
	strcat(oline,":");
	strncat(oline,parv[3],32);
	strcat(oline,":10\n");

	cfd=open(configfile,O_APPEND | O_WRONLY | O_SYNC);
	if (cfd<=0)
	{
		sendcmdto_one(&me, CMD_NOTICE, sptr,"%C :Erreur d'E/S", sptr);
		return 0;
	}

	arf=write(cfd,oline,strlen(oline));
	arf=close(cfd);

	sendto_allops(&me, SNO_OLDSNO, "Ajout d'une O-line globale (%s) pour %s!%s par %s",parv[3],parv[1],parv[2], cli_name(sptr));

	rehash(&me,1);

	return 0;
}

/*
 * mo_deloper - oper message handler
 *
 * parv[0] = sender
 * parv[1] = login
 * parv[2] = host
 */
int mo_deloper(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
	FILE *config;
	FILE *temp;
	char buff[65536];
	char buf2[65536];
	char *opernick,*operpass,*operhostmask;
	int found = 0;

	if(!IsAnAdmin(sptr))
		return send_reply(sptr, ERR_NOPRIVILEGES);

	if(parc < 3)
	{
		need_more_params(sptr, "DELOPER");
		sendcmdto_one(&me, CMD_NOTICE, sptr,
		  "%C :Syntax : DELOPER <login> <host>", sptr);
		return 0;
	}

	config = fopen(configfile,"r");
	remove("ircd.temp");
	temp = fopen("ircd.temp","w");
	if (config==NULL || temp==NULL)
	{
		sendcmdto_one(&me, CMD_NOTICE, sptr,"%C :Erreur d'E/S",sptr);
		return 0;
	}
	while (fgets(buff,65535,config))
	{
		if (!strncmp(buff,"O:",2))
		{
			strcpy(buf2,buff);
			strtok(buf2,":");
			operhostmask=strtok(NULL,":");
			operpass=strtok(NULL,":");
			opernick=strtok(NULL,":");
			if (operhostmask && operpass && opernick)
			{
				if ((!strcasecmp(opernick,parv[1])) && (!strcasecmp(operhostmask,parv[2])))
				{
					found=1;
					continue;
				}
			}
		}
		fprintf(temp,"%s",buff);
	}
	fclose(config);
	fclose(temp);
	rename("ircd.temp",configfile);
	if (found) sendcmdto_one(&me, CMD_NOTICE, sptr, "%C %s :O-line supprime", sptr, parv[1]);
	else
	{
		sendcmdto_one(&me, CMD_NOTICE, sptr,"%C %s :O-line non trouve", sptr, parv[1]);
		return 0;
	}


	sendto_allops(&me, SNO_OLDSNO, "%s a supprim la O-line global pour %s!%s",cli_name(sptr),parv[1],parv[2]);

	rehash(&me,1);

	return 0;
}
